diff --git a/dlls/windowscodecs/tests/stream.c b/dlls/windowscodecs/tests/stream.c
index 0dbcddf..bbe2b1c 100644
--- a/dlls/windowscodecs/tests/stream.c
+++ b/dlls/windowscodecs/tests/stream.c
@@ -737,8 +737,82 @@ static void test_StreamOnStreamRange(void)
     CoUninitialize();
 }
 
+void test_StreamOnStream(void)
+{
+    HGLOBAL hMem;
+    IWICImagingFactory *pFactory;
+    IStream *pStream;
+    IWICStream *pWicStream;
+    const BYTE CmpMem[] = {
+        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
+        0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,
+        0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
+        0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
+    };
+    BYTE Memory[64];
+    LARGE_INTEGER LargeNull;
+    ULARGE_INTEGER uNewPos;
+    ULONG uBytesRead, uBytesWritten;
+    HRESULT hr;
+    STATSTG Stats;
+
+    LargeNull.QuadPart = 0;
+
+    CoInitialize(NULL);
+
+    hr = CoCreateInstance(&CLSID_WICImagingFactory, NULL, CLSCTX_INPROC_SERVER, &IID_IWICImagingFactory, (void**)&pFactory);
+    if(FAILED(hr)) {
+        skip("CoCreateInstance returned with %#x, expected %#x\n", hr, S_OK);
+        return;
+    }
+
+    hr = IWICImagingFactory_CreateStream(pFactory, &pWicStream);
+    ok(hr == S_OK, "CreateStream returned with %#x, expected %#x\n", hr, S_OK);
+    if(FAILED(hr)) {
+        skip("Failed to create stream\n");
+        return;
+    }
+
+    /* InitializeFromIStream */
+    hMem = GlobalAlloc(GMEM_MOVEABLE, 16);
+    ok(hMem != NULL, "GlobalAlloc failed\n");
+    hr = CreateStreamOnHGlobal(hMem, TRUE, &pStream);
+
+    hr = IWICStream_InitializeFromIStream(pWicStream, pStream);
+    IStream_Release(pStream);
+    ok(hr == S_OK, "InitializeFromIStream returned with %#x, expected %#x\n", hr, S_OK);
+
+    hr = IStream_Stat(pWicStream, &Stats, STATFLAG_NONAME);
+    ok(hr == S_OK, "Stat returned %#x, expected %#x\n", hr, S_OK);
+    ok(Stats.cbSize.u.HighPart == 0 && Stats.cbSize.u.LowPart == 16, "Stat returned size (%u;%u)\n", Stats.cbSize.u.HighPart, Stats.cbSize.u.LowPart);
+
+    IStream_Write(pWicStream, CmpMem, sizeof(CmpMem), &uBytesWritten);
+    hr = IStream_Stat(pWicStream, &Stats, STATFLAG_NONAME);
+    ok(hr == S_OK, "Stat returned %#x, expected %#x\n", hr, S_OK);
+    ok(Stats.cbSize.u.HighPart == 0 && Stats.cbSize.u.LowPart == 64, "Stat returned size (%u;%u)\n", Stats.cbSize.u.HighPart, Stats.cbSize.u.LowPart);
+
+    IWICStream_Seek(pWicStream, LargeNull, STREAM_SEEK_SET, NULL);
+
+    /* Read */
+    hr = IWICStream_Read(pWicStream, Memory, sizeof(Memory), &uBytesRead);
+    ok(hr == S_OK, "Read returned with %#x, expected %#x\n", hr, S_OK);
+    if(SUCCEEDED(hr)) {
+        ok(uBytesRead == sizeof(Memory), "Read %u bytes, expected %u\n", uBytesRead, sizeof(Memory));
+        ok(memcmp(Memory, CmpMem, sizeof(Memory)) == 0, "Read returned invalid data!\n");
+
+        /* check whether the seek pointer has moved correctly */
+        IWICStream_Seek(pWicStream, LargeNull, STREAM_SEEK_CUR, &uNewPos);
+        ok(uNewPos.u.HighPart == 0 && uNewPos.u.LowPart == uBytesRead, "Seek cursor moved to position (%u;%u), expected (%u;%u)\n", uNewPos.u.HighPart, uNewPos.u.LowPart, 0, uBytesRead);
+    }
+
+    IWICStream_Release(pWicStream);
+    IWICStream_Release(pFactory);
+    CoUninitialize();
+}
+
 START_TEST(stream)
 {
     test_StreamOnMemory();
     test_StreamOnStreamRange();
+    test_StreamOnStream();
 }
